#!/usr/bin/perl
#
# getdisctoc.pl {username} {sessionnumber}
#
# read table of contents from CD using Paranoia.
# if successful, add tags to session:
#    NTRACKS     the number of tracks
#    OFFSETS     list of the track frame offsets
#    LENGTHS     list of the track frame lengths
#    TOTLENGTH   total length of disc in frames
#    CDDB        the cddb hash
#    MESSAGE     message indicating success
#    STATE       IDLE
#
# else, if not successful
#    MESSAGE     message indicating the problem
#    STATE       EMPTY
#    NTRACKS     0
#

$tunebot_root="XXX-the-tunebot-install-root";
$ses_directory=$tunebot_root."/data/loader";
$framesec=75;

if ($#ARGV != 1) {
    exit 1;
}
$usrnam=$ARGV[0];
$sesno=$ARGV[1];

sub readTunebotSettings {
    my ($i, $j);
    open (I, "$tunebot_root/settings");
    while ($i = <I>) {
	chomp $i;
	$j = index($i, "=");
	if ($j > 0) {
	    $settings{substr($i,0,$j)} = substr($i,$j+1);
	}
    }
    close(I);
}

readTunebotSettings;

$cdpara=$settings{"CDPARANOIA_PATH"};

#
# read the tracks toc from a CD using paranoia
# returns null if succeeded, else error message.
#
sub getcdtoc {
    # voodoo makes cdparanoia agree with cddb (?)
    my $voodoo=150;
    my $offsets=();
    my $lengths=();
    my $ntracks=0;
    my $i;
    my $tm;
    my $ts;

    open(I, "$cdpara -Q 2>&1 |") || return "Can't read CD";
    while (<I>) {
	if ($_ =~ /^ *[0-9]+\./) {
	    split;
	    @offsets=(@offsets, $_[3] + $voodoo);
	    @lengths=(@lengths, $_[1]);
	    ++$ntracks;
	}
    }
    close(I);
    if ($ntracks == 0) {
	return "Failed to read Table of Contents from CD";
    }
    addSesTag("NTRACKS=$ntracks");
    addSesTag("OFFSETS=".join(",", @offsets));
    addSesTag("LENGTHS=".join(",", @lengths));
    for ($i = 1; $i <= $ntracks; ++$i) {
	$tm = $lengths[$i-1] / $framesec;
	$ts = $tm % 60;
	$tm = $tm / 60;
	addSesTag(sprintf("TRACKTIME%d=%d:%02d", $i, $tm, $ts));
    }
    return;
}

#
# generate the CDDB 32-bit checksum for a CD given the tracks toc
#
sub gencddbsum {
    my $offsets=split /,/, getSesTag("OFFSETS");
    my $lengths=split /,/, getSesTag("LENGTHS");
    my $ntracks=getSesTag("NTRACKS");
    my $n = 0;
    my $i;
    my $j;

    # sum digits of begin offsets of tracks truncd to seconds
    for ($i = 0; $i < $ntracks; ++$i) {
	$j = int($offsets[$i] / $framesec);
	while ($j > 0) {
	    $n += $j % 10;
	    $j = int($j / 10);
	}
    }

    # total time (last offset plus length) minus begin offset
    $j = int(($offsets[$ntracks-1] + $lengths[$ntracks-1]) / $framesec);
    addSesTag("TOTLENGTH=$j");
    $j -= int($offsets[0] / $framesec);
    addSesTag("CDDB=".sprintf("%08x", ($n % 255)<<24 | ($j<<8) | $ntracks));
}

sub addSesTag {
    open (O, ">>$ses_directory/$usrnam.$sesno");
    print O @_, "\n";
    close(O);
    chmod 0666, "$ses_directory/$usrnam.$sesno";
}

sub getSesTag {
    my ($tag) = $_[0] . "=";
    my ($l) = length($tag);
    my $val;
    my $i;
    open (I, "$ses_directory/$usrnam.$sesno");
    while ($i = <I>) {
	chomp $i;
	if (substr($i, 0, $l) eq $tag) {
	    $val = $i;
	    $val =~ s/^[^=]*=//;
	}
    }
    close(I);
    return $val;
}

if ($err=getcdtoc()) {
    addSesTag("MESSAGE=$err");
    addSesTag("STATE=EMPTY");
    addSesTag("NTRACKS=0");
} else {
    gencddbsum();
    addSesTag("MESSAGE=Read Table of Contents from CD");
    addSesTag("STATE=IDLE");
}

exit 0;

