
/*
*  get_eq_coeffs.c
*
*  this is the start of a little program that would go through the
*  MP3 database and add equalizer tags to the tag file for each track.
*  this would let the mp3 player adjust the volume output (e.g. with the
*  mpg123 -f option) track-by-track without having to modify the actual
*  MP3 data (in case we decide we didn't do it right the first time).
*
*  my plan was to make a list of coefficients that represented the envelope
*  that 90%, 99%, 99.9%, ... of the samples fall inside.  also, the 100%
*  level.  this should let you intelligently crank up the volume to 11 if
*  you want, but not to, for example, 12.
*
*  right now it does the hard part but doesn't write the tags.  huh.
*/

#include <stdio.h>
#include <fcntl.h>

#define MPGPLAYER "/usr/bin/mpg123"

int
already_have_eq_tags(
    char *file )
{
    FILE *ff;
    char buf[1024];

    if (!(ff = fopen(file, "r"))) {
	fprintf(stderr, "%s: can't read\n", file);
	return 1;
    }
    while (fgets(buf, sizeof(buf), ff)) {
	if (!strncmp(buf, "E=", 2)) {
	    fclose(ff);
	    return 1;
	}
    }
    fclose(ff);
    return 0;
}

get_eq_vals(
    int fd,
    char *obuf )
{
    int i, j;
    int n;
    int v;
    int nsamples = 0;
    int cumul;
    int q;
    char *sep;
    int amps[256];
    unsigned char buf[4096];
    double fits[] = { 0.9, 0.99, 0.999, 0.9999, 0.99999, 0.999999, 0.9999999 };

    obuf[0] = 0;

    for (i = 0; i < sizeof(amps)/sizeof(amps[0]); i++)
	amps[i] = 0;

    while ((n = read(fd, buf, sizeof(buf))) > 3) {
	for (i = 0; i < n - 3; i += 4) {
	    nsamples += 2;
	    v = buf[i] + 256 * buf[i + 1];
	    if (v & 0x8000)
		v ^= 0xffff;
	    amps[v / (32768/(sizeof(amps)/sizeof(amps[0])))]++;
	    v = buf[i + 2] + 256 * buf[i + 3];
	    if (v & 0x8000)
		v ^= 0xffff;
	    amps[v / (32768/(sizeof(amps)/sizeof(amps[0])))]++;
	}
    }

#if 0
    cumul = 0;
    for (i = 0; i < sizeof(amps)/sizeof(amps[0]); i++) {
	cumul += amps[i];
	printf("%3d %6.2f %6.2f\n", i, amps[i] * 100.0 / nsamples,
		cumul * 100.0 / nsamples);
    }
#endif

    cumul = 0;
    for (i = 0; i < sizeof(amps)/sizeof(amps[0]); i++) {
	cumul += amps[i];
	if (cumul == nsamples) {
	    break;
	}
    }
    sprintf(obuf, "EQPK=%.3f\nEQFIT=",
	    ((double)i + 1) / (sizeof(amps)/sizeof(amps[0])));
    q = 0;
    cumul = 0;
    sep = "";
    for (i = 0; i < sizeof(amps)/sizeof(amps[0]); i++) {
	cumul += amps[i];
	while ((double)cumul / nsamples >= fits[q]) {
	    sprintf(obuf + strlen(obuf), "%s%.3f", sep,
		    ((double)i + 1) / (sizeof(amps)/sizeof(amps[0])));
	    sep = ",";
	    if (++q == sizeof(fits)/sizeof(fits[0]))
		break;
	}
	if (q == sizeof(fits)/sizeof(fits[0]))
	    break;
    }
    sprintf(obuf + strlen(obuf), "\n");
}

int
generate_tags(
    char *file,
    char *output )
{
    char *av[10];
    int i;
    int pfd[2];
    char obuf[200];

    av[0] = MPGPLAYER;
    av[1] = "-q";
    av[2] = "-s";
    av[3] = file;
    av[4] = 0;
    if (pipe(pfd)) {
	perror("pipe");
	return 1;
    }
    i = fork();
    if (i == -1) {
	perror("fork");
	close(pfd[0]);
	close(pfd[1]);
	return 1;
    }
    if (!i) {
	close(pfd[0]);
	dup2(pfd[1], 1);
	execv(av[0], av);
	perror("execv");
	exit(0);
    }
    close(pfd[1]);
#if 0
    printf("pfd[0,1]=%d %d\n", pfd[0], pfd[1]);
#endif
    get_eq_vals(pfd[0], obuf);
    close(pfd[0]);
    wait(0);
    printf("%s\n", obuf);
}

int
main(
    int argc,
    char **argv )
{
    int i;
    char mp3file[1024];
    char tagfile[1024];

    while (fgets(mp3file, sizeof(mp3file)-4, stdin)) {
	i = strlen(mp3file);
	if (i > 0 && mp3file[i - 1] == '\n')
	    mp3file[--i] = 0;
	if (i > 3 && mp3file[i - 4] == '.' && mp3file[i - 3] == 'm'
		&& mp3file[i - 2] == 'p' && mp3file[i - 1] == '3')
	    i -= 4;
	strcpy(tagfile, mp3file);
	tagfile[i] = '.';
	tagfile[i + 1] = 't';
	tagfile[i + 2] = 0;
	if (access(mp3file, R_OK)) {
	    perror(mp3file);
	    continue;
	}
	if (access(tagfile, R_OK)) {
	    perror(tagfile);
	    continue;
	}
	if (already_have_eq_tags(tagfile)) {
	    printf("%s: skipping, already has tags\n", tagfile);
	    continue;
	}
	printf("%s: adding tags\n", tagfile);
	generate_tags(mp3file, tagfile);
    }
}

